@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2009 GraphicsMagick Group
+% Copyright (C) 2003 - 2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -129,14 +129,16 @@ static unsigned int IsMETA(const unsigned char *magick,const size_t length)
 
 typedef struct _html_code
 {
-  short
+  unsigned short
     len;
+
   const char
     *code,
     val;
+
 } html_code;
 
-static html_code html_codes[] = {
+static html_code const html_codes[] = {
 #ifdef HANDLE_GT_LT
   { 4,"&lt;",'<' },
   { 4,"&gt;",'>' },
@@ -176,32 +178,39 @@ static int stringnicmp(const char *p,const char *q,size_t n)
     p++;
     q++;
   }
-  return(toupper(*p)-toupper(*q));
+  return(toupper((int) *p)-toupper((int) *q));
 }
 
-static int convertHTMLcodes(char *s, int len)
+static size_t convertHTMLcodes(char *s, const size_t len)
 {
-  if (len <=0 || s==(char*)NULL || *s=='\0')
+  int val;
+
+  if ((len == 0) || (s == (char*) NULL) || (*s=='\0'))
     return 0;
 
-  if (s[1] == '#')
-    {
-      int val, o;
+  /*
+    Check for hex or decimal character escape.
 
-      if (sscanf(s,"&#%d;",&val) == 1)
-      {
-        o = 3;
-        while (s[o] != ';')
+    e.g. hex: "&#xA0;", decimal "&#160;"
+
+    Note that scanf is expected to parse the number in hex or decimal
+    and that it does not require the terminating ';' to appear in
+    order to parse the numeric argument.
+  */
+  if ((len > 3) && (s[1] == '#') && (strchr(s,';') != (char *) NULL) &&
+      (sscanf(s,"&#%d;",&val) == 1))
+    {
+      size_t o = 3;
+      while (s[o] != ';')
         {
           o++;
           if (o > 5)
             break;
         }
-        if (o < 6)
-          (void) strcpy(s+1, s+1+o);
-        *s = val;
-        return o;
-      }
+      if (o < 6)
+        (void) memmove(s+1,s+1+o,strlen(s+1+o)+1);
+      *s = val;
+      return o;
     }
   else
     {
@@ -214,7 +223,7 @@ static int convertHTMLcodes(char *s, int len)
         if (html_codes[i].len <= len)
           if (stringnicmp(s, html_codes[i].code, (size_t) (html_codes[i].len)) == 0)
             {
-              (void) strcpy(s+1, s+html_codes[i].len);
+              (void) memmove(s+1, s+html_codes[i].len, strlen(s+html_codes[i].len)+1);
               *s = html_codes[i].val;
               return html_codes[i].len-1;
             }
@@ -281,10 +290,10 @@ static long parse8BIM(Image *ifile, Image *ofile)
   char
     brkused,
     quoted,
-    *line,
-    *token,
-    *newstr,
-    *name;
+    *line = (char *) NULL,
+    *token = (char *) NULL,
+    *newstr = (char *) NULL,
+    *name = (char *) NULL;
 
   int
     state,
@@ -313,7 +322,8 @@ static long parse8BIM(Image *ifile, Image *ofile)
   dataset = 0;
   recnum = 0;
   line = MagickAllocateMemory(char *,inputlen);
-  name = token = (char *)NULL;
+  if (line == (char *) NULL)
+    goto parse8BIM_failure;
   savedpos = 0;
   while(super_fgets(&line,&inputlen,ifile)!=NULL)
   {
@@ -321,7 +331,11 @@ static long parse8BIM(Image *ifile, Image *ofile)
     next=0;
 
     token = MagickAllocateMemory(char *,inputlen);
+    if (token == (char *) NULL)
+      goto parse8BIM_failure;
     newstr = MagickAllocateMemory(char *,inputlen);
+    if (newstr == (char *) NULL)
+      goto parse8BIM_failure;
     while (Tokenizer(&token_info, 0, token, inputlen, line,
           (char *) "", (char *) "=",
       (char *) "\"", 0, &brkused,&next,&quoted)==0)
@@ -347,15 +361,16 @@ static long parse8BIM(Image *ifile, Image *ofile)
                 if (strcmp(newstr,"8BIM")==0)
                   dataset = 255;
                 else
-                  dataset = (unsigned char) atoi(newstr);
+                  dataset = (unsigned char) MagickAtoI(newstr);
                 break;
               case 1:
-                recnum = atoi(newstr);
+                recnum = MagickAtoI(newstr);
                 break;
               case 2:
                 name = MagickAllocateMemory(char *,strlen(newstr)+1);
-                if (name)
-                  (void) strcpy(name,newstr);
+                if (name == (char *) NULL)
+                  goto parse8BIM_failure;
+                (void) strcpy(name,newstr);
                 break;
             }
             state++;
@@ -367,7 +382,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
             int
               next;
 
-            unsigned long
+            size_t
               len;
 
             char
@@ -375,7 +390,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
               quoted;
 
             next=0;
-            len = strlen(token);
+            len = (unsigned long) strlen(token);
             while (Tokenizer(&token_info,0, newstr, inputlen, token, (char *) "",
               (char *) "&", (char *) "", 0, &brkused, &next, &quoted)==0)
             {
@@ -400,6 +415,8 @@ static long parse8BIM(Image *ifile, Image *ofile)
                   {
                     long diff = outputlen - savedolen;
                     currentpos = TellBlob(ofile);
+                    if (currentpos < 0)
+                      goto parse8BIM_failure;
                     (void) SeekBlob(ofile,savedpos,SEEK_SET);
                     (void) WriteBlobMSBLong(ofile,diff);
                     (void) SeekBlob(ofile,currentpos,SEEK_SET);
@@ -414,7 +431,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
                 (void) WriteBlobMSBShort(ofile,recnum);
                 outputlen += 6;
                 if (name)
-                  nlen = strlen(name);
+                  nlen = (unsigned char) strlen(name);
                 (void) WriteBlobByte(ofile,nlen);
                 outputlen++;
                 for (i=0; i<nlen; i++)
@@ -427,13 +444,13 @@ static long parse8BIM(Image *ifile, Image *ofile)
                   }
                 if (recnum != IPTC_ID)
                   {
-                    (void) WriteBlobMSBLong(ofile, len);
+                    (void) WriteBlobMSBLong(ofile, (const magick_uint32_t) len);
                     outputlen += 4;
 
                     next=0;
                     outputlen += len;
                     while (len--)
-                      (void) WriteBlobByte(ofile,token[next++]);
+                      (void) WriteBlobByte(ofile,token[next++]); /* boom */
 
                     if (outputlen & 1)
                       {
@@ -445,6 +462,8 @@ static long parse8BIM(Image *ifile, Image *ofile)
                   {
                     /* patch in a fake length for now and fix it later */
                     savedpos = TellBlob(ofile);
+                    if (savedpos < 0)
+                      goto parse8BIM_failure;
                     (void) WriteBlobMSBLong(ofile,0xFFFFFFFFUL);
                     outputlen += 4;
                     savedolen = outputlen;
@@ -457,7 +476,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
                     (void) WriteBlobByte(ofile,0x1c);
                     (void) WriteBlobByte(ofile,dataset);
                     (void) WriteBlobByte(ofile,recnum & 255);
-                    (void) WriteBlobMSBShort(ofile,len);
+                    (void) WriteBlobMSBShort(ofile,(const magick_uint16_t) len);
                     outputlen += 5;
                     next=0;
                     outputlen += len;
@@ -478,12 +497,22 @@ static long parse8BIM(Image *ifile, Image *ofile)
       long diff = outputlen - savedolen;
 
       currentpos = TellBlob(ofile);
+      if (currentpos < 0)
+        goto parse8BIM_failure;
+
       (void) SeekBlob(ofile,savedpos,SEEK_SET);
       (void) WriteBlobMSBLong(ofile,diff);
       (void) SeekBlob(ofile,currentpos,SEEK_SET);
       savedolen = 0L;
     }
   return outputlen;
+
+ parse8BIM_failure:
+  MagickFreeMemory(token);
+  MagickFreeMemory(newstr);
+  MagickFreeMemory(name);
+  MagickFreeMemory(line);
+  return 0L;
 }
 
 static char *super_fgets_w(char **b, size_t *blen, Image *file)
@@ -542,10 +571,10 @@ static long parse8BIMW(Image *ifile, Image *ofile)
   char
     brkused,
     quoted,
-    *line,
-    *token,
-    *newstr,
-    *name;
+    *line = (char *) NULL,
+    *token = (char *) NULL,
+    *newstr = (char *) NULL,
+    *name = (char *) NULL;
 
   int
     state,
@@ -574,6 +603,8 @@ static long parse8BIMW(Image *ifile, Image *ofile)
   dataset = 0;
   recnum = 0;
   line = MagickAllocateMemory(char *,inputlen);
+  if (line == (char *) NULL)
+    goto parse8BIMW_failure;
   name = token = (char *)NULL;
   savedpos = 0;
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -586,7 +617,11 @@ static long parse8BIMW(Image *ifile, Image *ofile)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                           "META CODER Parse8BIM: %s (%lu)",line, (unsigned long) inputlen);
     token = MagickAllocateMemory(char *,inputlen);
+    if (token == (char *) NULL)
+      goto parse8BIMW_failure;
     newstr = MagickAllocateMemory(char *,inputlen);
+    if (newstr == (char *) NULL)
+      goto parse8BIMW_failure;
     while (Tokenizer(&token_info, 0, token, inputlen, line,
           (char *) "", (char *) "=",
       (char *) "\"", 0, &brkused,&next,&quoted)==0)
@@ -612,15 +647,16 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                 if (strcmp(newstr,"8BIM")==0)
                   dataset = 255;
                 else
-                  dataset = (unsigned char) atoi(newstr);
+                  dataset = (unsigned char) MagickAtoI(newstr);
                 break;
               case 1:
-                recnum = atoi(newstr);
+                recnum = MagickAtoI(newstr);
                 break;
               case 2:
                 name = MagickAllocateMemory(char *,strlen(newstr)+1);
-                if (name)
-                  (void) strcpy(name,newstr);
+                if (name == (char *) NULL)
+                  goto parse8BIMW_failure;
+                (void) strcpy(name,newstr);
                 break;
             }
             state++;
@@ -632,7 +668,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
             int
               next;
 
-            unsigned long
+            size_t
               len;
 
             char
@@ -640,7 +676,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
               quoted;
 
             next=0;
-            len = strlen(token);
+            len = (unsigned long) strlen(token);
             while (Tokenizer(&token_info,0, newstr, inputlen, token, (char *) "",
               (char *) "&", (char *) "", 0, &brkused, &next, &quoted)==0)
             {
@@ -665,6 +701,8 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                   {
                     long diff = outputlen - savedolen;
                     currentpos = TellBlob(ofile);
+                    if (currentpos < 0)
+                      goto parse8BIMW_failure;
                     (void) SeekBlob(ofile,savedpos,SEEK_SET);
                     (void) WriteBlobMSBLong(ofile,diff);
                     (void) SeekBlob(ofile,currentpos,SEEK_SET);
@@ -679,7 +717,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                 (void) WriteBlobMSBShort(ofile,recnum);
                 outputlen += 6;
                 if (name)
-                  nlen = strlen(name);
+                  nlen = (unsigned char) strlen(name);
                 (void) WriteBlobByte(ofile,nlen);
                 outputlen++;
                 for (i=0; i<nlen; i++)
@@ -692,7 +730,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                   }
                 if (recnum != IPTC_ID)
                   {
-                    (void) WriteBlobMSBLong(ofile, len);
+                    (void) WriteBlobMSBLong(ofile, (const magick_uint32_t) len);
                     outputlen += 4;
 
                     next=0;
@@ -710,6 +748,8 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                   {
                     /* patch in a fake length for now and fix it later */
                     savedpos = TellBlob(ofile);
+                    if (savedpos < 0)
+                      goto parse8BIMW_failure;
                     (void) WriteBlobMSBLong(ofile,0xFFFFFFFFUL);
                     outputlen += 4;
                     savedolen = outputlen;
@@ -722,10 +762,10 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                     (void) WriteBlobByte(ofile,0x1c);
                     (void) WriteBlobByte(ofile,dataset);
                     (void) WriteBlobByte(ofile,recnum & 255);
-                    (void) WriteBlobMSBShort(ofile,len);
+                    (void) WriteBlobMSBShort(ofile,(const magick_uint16_t) len);
                     outputlen += 5;
                     next=0;
-                    outputlen += len;
+                    outputlen += (long) len;
                     while (len--)
                       (void) WriteBlobByte(ofile,token[next++]);
                   }
@@ -743,12 +783,21 @@ static long parse8BIMW(Image *ifile, Image *ofile)
       long diff = outputlen - savedolen;
 
       currentpos = TellBlob(ofile);
+      if (currentpos < 0)
+        goto parse8BIMW_failure;
       (void) SeekBlob(ofile,savedpos,SEEK_SET);
       (void) WriteBlobMSBLong(ofile,diff);
       (void) SeekBlob(ofile,currentpos,SEEK_SET);
       savedolen = 0L;
     }
   return outputlen;
+
+ parse8BIMW_failure:
+  MagickFreeMemory(token);
+  MagickFreeMemory(newstr);
+  MagickFreeMemory(name);
+  MagickFreeMemory(line);
+  return 0L;
 }
 
 /* some defines for the different JPEG block types */
@@ -908,8 +957,6 @@ static int jpeg_skip_till_marker(Image *ifile, int marker)
 }
 #endif
 
-static char psheader[] = "\xFF\xED\0\0Photoshop 3.0\08BIM\x04\x04\0\0\0\0";
-
 /* Embed binary IPTC data into a JPEG image. */
 static int jpeg_embed(Image *ifile, Image *ofile, Image *iptc)
 {
@@ -952,9 +999,12 @@ static int jpeg_embed(Image *ifile, Image *ofile, Image *iptc)
 
         if (iptc != (Image *)NULL)
           {
+            static const char psheader_base[] = "\xFF\xED\0\0Photoshop 3.0\08BIM\x04\x04\0\0\0\0";
+            char psheader[sizeof(psheader_base)];
             len=GetBlobSize(iptc);
             if (len & 1)
               len++; /* make the length even */
+            (void) strlcpy(psheader,psheader_base,sizeof(psheader));
             psheader[ 2 ] = (len+16)>>8;
             psheader[ 3 ] = (len+16)&0xff;
             for (inx = 0; inx < 18; inx++)
@@ -1151,9 +1201,9 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             }
           AttachBlob(iptc->blob,pinfo->info,pinfo->length);
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                                "META CODER APP1JPEG embed: 0x%08lx (%ld)",
-                                (unsigned long) pinfo->info,
-                                (long) pinfo->length);
+                                "META CODER APP1JPEG embed: %p (%ld)",
+                                pinfo->info, /* unsigned char * */
+                                (long) pinfo->length); /* size_t */
           result=jpeg_embed(image,buff,iptc);
           DetachBlob(iptc->blob);
           DestroyImage(iptc);
@@ -1250,7 +1300,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
 
       DestroyImage(buff);
     }
-  if (LocaleCompare(image_info->magick,"IPTC") == 0)
+  if (LocaleNCompare(image_info->magick,"IPTC",4) == 0)
     {
       buff=AllocateImage((ImageInfo *) NULL);
       if (buff == (Image *) NULL)
@@ -1260,7 +1310,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         {
           DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
-            image)
+                               image);
         }
       AttachBlob(buff->blob,blob,length);
       /* write out the header - length field patched below */
@@ -1286,14 +1336,21 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         }
 
       /* subtract off the length of the 8BIM stuff */
+      blob=GetBlobStreamData(buff); /* blob may be realloced */
+      if ((blob == (const unsigned char *) NULL) || (GetBlobSize(buff) < 12))
+        {
+          DestroyImage(buff);
+          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
+                               image);
+        }
       length=GetBlobSize(buff)-12;
-      blob[10]=length >> 8;
-      blob[11]=length & 0xff;
+      blob[10]=(unsigned char) ((length >> 8) & 0xff);
+      blob[11]=(unsigned char) (length & 0xff);
       blob=GetBlobStreamData(buff);
       length=GetBlobSize(buff);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                            "Store IPTC profile, size %lu bytes",
-                            (unsigned long) length);
+                            "Store IPTC profile, size %" MAGICK_SIZE_T_F
+                            "u bytes",(MAGICK_SIZE_T) length);
       (void) SetImageProfile(image,"IPTC",blob,length);
       DetachBlob(buff->blob);
       MagickFreeMemory(blob)
@@ -1522,9 +1579,9 @@ ModuleExport void UnregisterMETAImage(void)
 %
 */
 
-static long GetIPTCStream(const unsigned char *blob, size_t blob_length, size_t *offset)
+static size_t GetIPTCStream(const unsigned char *blob, size_t blob_length, size_t *offset)
 {
-  long
+  size_t
     info_length;
 
   register long
@@ -1534,13 +1591,12 @@ static long GetIPTCStream(const unsigned char *blob, size_t blob_length, size_t
     *p;
 
   unsigned char
-    buffer[4],
     c;
 
   unsigned int
     marker;
 
-  unsigned long
+  size_t
     tag_length,
     blob_remaining;
     
@@ -1583,15 +1639,17 @@ static long GetIPTCStream(const unsigned char *blob, size_t blob_length, size_t
     p+=4;
     blob_remaining-=4;
     if (tag_length > blob_remaining) break;
-
     /* Check whether we have the IPTC tag data */
     if (marker == IPTC_ID)
       {
         /* All looks good so return IPTC block "as is" */
         *offset=(unsigned long)(p-blob);
-        return blob_remaining;
+        return tag_length;
       }
-      
+
+    /* Allow for padding of data to even size */
+    if (tag_length & 1) tag_length++;
+
     /* Skip unwanted data */
     p+=tag_length;
     blob_remaining-=tag_length;
@@ -1660,19 +1718,21 @@ iptc_find:
     info_length++;
     if (c & (unsigned char) 0x80)
       {
-        for (i=0; i < 4; i++)
+	/* long format */
+	tag_length=0;
+	for (i=0; i < 4; i++)
         {
-          buffer[i]=(*p++);
+	  tag_length <<= 8;
+          tag_length |= (*p++);
           blob_length--;
           if (blob_length == 0)
             break;
           info_length++;
         }
-        tag_length=(((long) buffer[0]) << 24) | (((long) buffer[1]) << 16) |
-          (((long) buffer[2]) << 8) | (((long) buffer[3]));
       }
     else
       {
+	/* short format */
         tag_length=((long) c) << 8;
         c=(*p++);
         blob_length--;
@@ -1738,64 +1798,64 @@ typedef struct _tag_spec
   short
     id;
 
-  char
+  const char
     *name;
 } tag_spec;
 
-static tag_spec tags[] = {
-  { 5, (char *) "Image Name" },
-  { 7, (char *) "Edit Status" },
-  { 10, (char *) "Priority" },
-  { 15, (char *) "Category" },
-  { 20, (char *) "Supplemental Category" },
-  { 22, (char *) "Fixture Identifier" },
-  { 25, (char *) "Keyword" },
-  { 30, (char *) "Release Date" },
-  { 35, (char *) "Release Time" },
-  { 40, (char *) "Special Instructions" },
-  { 45, (char *) "Reference Service" },
-  { 47, (char *) "Reference Date" },
-  { 50, (char *) "Reference Number" },
-  { 55, (char *) "Created Date" },
-  { 60, (char *) "Created Time" },
-  { 65, (char *) "Originating Program" },
-  { 70, (char *) "Program Version" },
-  { 75, (char *) "Object Cycle" },
-  { 80, (char *) "Byline" },
-  { 85, (char *) "Byline Title" },
-  { 90, (char *) "City" },
-  { 95, (char *) "Province State" },
-  { 100, (char *) "Country Code" },
-  { 101, (char *) "Country" },
-  { 103, (char *) "Original Transmission Reference" },
-  { 105, (char *) "Headline" },
-  { 110, (char *) "Credit" },
-  { 115, (char *) "Source" },
-  { 116, (char *) "Copyright String" },
-  { 120, (char *) "Caption" },
-  { 121, (char *) "Image Orientation" },
-  { 122, (char *) "Caption Writer" },
-  { 131, (char *) "Local Caption" },
-  { 200, (char *) "Custom Field 1" },
-  { 201, (char *) "Custom Field 2" },
-  { 202, (char *) "Custom Field 3" },
-  { 203, (char *) "Custom Field 4" },
-  { 204, (char *) "Custom Field 5" },
-  { 205, (char *) "Custom Field 6" },
-  { 206, (char *) "Custom Field 7" },
-  { 207, (char *) "Custom Field 8" },
-  { 208, (char *) "Custom Field 9" },
-  { 209, (char *) "Custom Field 10" },
-  { 210, (char *) "Custom Field 11" },
-  { 211, (char *) "Custom Field 12" },
-  { 212, (char *) "Custom Field 13" },
-  { 213, (char *) "Custom Field 14" },
-  { 214, (char *) "Custom Field 15" },
-  { 215, (char *) "Custom Field 16" },
-  { 216, (char *) "Custom Field 17" },
-  { 217, (char *) "Custom Field 18" },
-  { 218, (char *) "Custom Field 19" },
-  { 219, (char *) "Custom Field 20" }
+static const tag_spec tags[] = {
+  { 5, "Image Name" },
+  { 7, "Edit Status" },
+  { 10, "Priority" },
+  { 15, "Category" },
+  { 20, "Supplemental Category" },
+  { 22, "Fixture Identifier" },
+  { 25, "Keyword" },
+  { 30, "Release Date" },
+  { 35, "Release Time" },
+  { 40, "Special Instructions" },
+  { 45, "Reference Service" },
+  { 47, "Reference Date" },
+  { 50, "Reference Number" },
+  { 55, "Created Date" },
+  { 60, "Created Time" },
+  { 65, "Originating Program" },
+  { 70, "Program Version" },
+  { 75, "Object Cycle" },
+  { 80, "Byline" },
+  { 85, "Byline Title" },
+  { 90, "City" },
+  { 95, "Province State" },
+  { 100, "Country Code" },
+  { 101, "Country" },
+  { 103, "Original Transmission Reference" },
+  { 105, "Headline" },
+  { 110, "Credit" },
+  { 115, "Source" },
+  { 116, "Copyright String" },
+  { 120, "Caption" },
+  { 121, "Image Orientation" },
+  { 122, "Caption Writer" },
+  { 131, "Local Caption" },
+  { 200, "Custom Field 1" },
+  { 201, "Custom Field 2" },
+  { 202, "Custom Field 3" },
+  { 203, "Custom Field 4" },
+  { 204, "Custom Field 5" },
+  { 205, "Custom Field 6" },
+  { 206, "Custom Field 7" },
+  { 207, "Custom Field 8" },
+  { 208, "Custom Field 9" },
+  { 209, "Custom Field 10" },
+  { 210, "Custom Field 11" },
+  { 211, "Custom Field 12" },
+  { 212, "Custom Field 13" },
+  { 213, "Custom Field 14" },
+  { 214, "Custom Field 15" },
+  { 215, "Custom Field 16" },
+  { 216, "Custom Field 17" },
+  { 217, "Custom Field 18" },
+  { 218, "Custom Field 19" },
+  { 219, "Custom Field 20" }
 };
 
 static int formatIPTC(Image *ifile, Image *ofile)
@@ -1837,9 +1897,14 @@ static int formatIPTC(Image *ifile, Image *ofile)
     else
       {
         if (foundiptc)
-          return -1;
+          {
+            return -1;
+          }
         else
-          continue;
+          {
+            c = ReadBlobByte(ifile);
+            continue;
+          }
       }
 
     /* we found the 0x1c tag and now grab the dataset and record number tags */
@@ -1885,7 +1950,11 @@ static int formatIPTC(Image *ifile, Image *ofile)
     for (tagindx=0; tagindx<taglen; tagindx++)
     {
       c=ReadBlobByte(ifile);
-      if (c == EOF) return -1;
+      if (c == EOF)
+        {
+          MagickFreeMemory(str);
+          return -1;
+        }
       str[tagindx] = (unsigned char) c;
     }
     str[taglen] = 0;
@@ -2008,7 +2077,11 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, long len)
     for (tagindx=0; tagindx<taglen; tagindx++)
     {
       c = *s++; len--;
-      if (len < 0) return -1;
+      if (len < 0)
+        {
+          MagickFreeMemory(str);
+          return -1;
+        }
       str[tagindx] = (unsigned char) c;
     }
     str[ taglen ] = 0;
@@ -2027,6 +2100,13 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, long len)
   return tagsfound;
 }
 
+#define Format8BIMLiberate()                    \
+  do                                            \
+    {                                           \
+      MagickFreeMemory(PString);                \
+      MagickFreeMemory(str);                    \
+    } while(0);
+
 static int format8BIM(Image *ifile, Image *ofile)
 {
   char
@@ -2042,109 +2122,138 @@ static int format8BIM(Image *ifile, Image *ofile)
     c;
 
   unsigned char
-    *PString,
-    *str;
+    *PString=0,
+    *str=0;
 
   resCount = 0;
 
   c =ReadBlobByte(ifile);
   while (c != EOF)
-  {
-    if (c == '8')
-      {
-        unsigned char
-          buffer[5];
+    {
+      if (c == '8')
+        {
+          unsigned char
+            buffer[5];
 
-        buffer[0] = (unsigned char) c;
-        for (i=1; i<4; i++)
+          buffer[0] = (unsigned char) c;
+          for (i=1; i<4; i++)
+            {
+              c=ReadBlobByte(ifile);
+              if (c == EOF)
+                {
+                  Format8BIMLiberate();
+                  return -1;
+                }
+              buffer[i] = (unsigned char) c;
+            }
+          buffer[4] = 0;
+          if (strcmp((const char *)buffer, "8BIM") != 0)
+            continue;
+        }
+      else
         {
           c=ReadBlobByte(ifile);
-          if (c == EOF) return -1;
-          buffer[i] = (unsigned char) c;
-        }
-        buffer[4] = 0;
-        if (strcmp((const char *)buffer, "8BIM") != 0)
           continue;
-      }
-    else
-      {
-        c=ReadBlobByte(ifile);
-        continue;
-      }
+        }
 
-    /* we found the OSType (8BIM) and now grab the ID, PString, and Size fields */
-    ID = ReadBlobMSBShort(ifile);
-    if (ID < 0) return -1;
-    {
-      unsigned char
-        plen;
+      /* we found the OSType (8BIM) and now grab the ID, PString, and Size fields */
+      ID = ReadBlobMSBShort(ifile);
+      if (ID < 0)
+        {
+          Format8BIMLiberate();
+          return -1;
+        }
+      {
+        unsigned char
+          plen;
 
-      c=ReadBlobByte(ifile);
-      if (c == EOF) return -1;
-      plen = (unsigned char) c;
-      PString=MagickAllocateMemory(unsigned char *,(unsigned int) (plen+1));
-      if (PString == (unsigned char *) NULL)
-      {
-        (void) printf("MemoryAllocationFailed");
-        return 0;
-      }
-      for (i=0; i<plen; i++)
-      {
         c=ReadBlobByte(ifile);
-        if (c == EOF) return -1;
-        PString[i] = (unsigned char) c;
-      }
-      PString[ plen ] = 0;
-      if (!(plen&1))
-      {
-        c=ReadBlobByte(ifile);
-        if (c == EOF) return -1;
-      }
-    }
-    Size = ReadBlobMSBLong(ifile);
-    if (Size < 0) return -1;
-    /* make a buffer to hold the data and snag it from the input stream */
-    str=MagickAllocateMemory(unsigned char *,(size_t) Size);
-    if (str == (unsigned char *) NULL)
-      {
-        (void) printf("MemoryAllocationFailed");
-        return 0;
-      }
-    for (i=0; i<Size; i++)
-    {
-      c=ReadBlobByte(ifile);
-      if (c == EOF) return -1;
-      str[i] = (unsigned char) c;
-    }
-
-    /* we currently skip thumbnails, since it does not make
-     * any sense preserving them in a real world application
-     */
-    if (ID != THUMBNAIL_ID)
-      {
-        /* now finish up by formatting this binary data into
-         * ASCII equivalent
-         */
-        if (strlen((const char *) PString) != 0)
-          FormatString(temp, "8BIM#%d#%s=", ID, PString);
-        else
-          FormatString(temp, "8BIM#%d=", ID);
-        (void) WriteBlobString(ofile,temp);
-        if (ID == IPTC_ID)
+        if (c == EOF)
           {
-            formatString(ofile, "IPTC", 4);
-            (void) formatIPTCfromBuffer(ofile, (char *)str, (long) Size);
+            Format8BIMLiberate();
+            return -1;
+          }
+        plen = (unsigned char) c;
+        PString=MagickAllocateMemory(unsigned char *,(unsigned int) (plen+1));
+        if (PString == (unsigned char *) NULL)
+          {
+            (void) printf("MemoryAllocationFailed");
+            Format8BIMLiberate();
+            return 0;
+          }
+        for (i=0; i<plen; i++)
+          {
+            c=ReadBlobByte(ifile);
+            if (c == EOF)
+              {
+                Format8BIMLiberate();
+                return -1;
+              }
+            PString[i] = (unsigned char) c;
+          }
+        PString[ plen ] = 0;
+        if (!(plen&1))
+          {
+            c=ReadBlobByte(ifile);
+            if (c == EOF)
+              {
+                Format8BIMLiberate();
+                return -1;
+              }
           }
-        else
-          formatString(ofile, (char *)str, (long) Size);
       }
-    MagickFreeMemory(str);
-    MagickFreeMemory(PString);
+      Size = ReadBlobMSBLong(ifile);
+      if (Size < 0)
+        {
+          Format8BIMLiberate();
+          return -1;
+        }
+      /* make a buffer to hold the data and snag it from the input stream */
+      str=MagickAllocateMemory(unsigned char *,(size_t) Size);
+      if (str == (unsigned char *) NULL)
+        {
+          (void) printf("MemoryAllocationFailed");
+          Format8BIMLiberate();
+          return 0;
+        }
+      for (i=0; i<Size; i++)
+        {
+          c=ReadBlobByte(ifile);
+          if (c == EOF)
+            {
+              Format8BIMLiberate();
+              return -1;
+            }
+          str[i] = (unsigned char) c;
+        }
 
-    resCount++;
+      /* we currently skip thumbnails, since it does not make
+       * any sense preserving them in a real world application
+       */
+      if (ID != THUMBNAIL_ID)
+        {
+          /* now finish up by formatting this binary data into
+           * ASCII equivalent
+           */
+          if (strlen((const char *) PString) != 0)
+            FormatString(temp, "8BIM#%d#%s=", ID, PString);
+          else
+            FormatString(temp, "8BIM#%d=", ID);
+          (void) WriteBlobString(ofile,temp);
+          if (ID == IPTC_ID)
+            {
+              formatString(ofile, "IPTC", 4);
+              (void) formatIPTCfromBuffer(ofile, (char *)str, (long) Size);
+            }
+          else
+            formatString(ofile, (char *)str, (long) Size);
+        }
+      Format8BIMLiberate();
 
-    c=ReadBlobByte(ifile);
-  }
+      resCount++;
+
+      c=ReadBlobByte(ifile);
+    }
   return resCount;
 }
 
