@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -37,13 +37,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/composite.h"
 #include "magick/constitute.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/profile.h"
 #include "magick/tempfile.h"
 #include "magick/transform.h"
@@ -52,36 +52,39 @@
 /*
   GraphicsMagick Macintosh PICT Methods.
 */
-#define ReadPixmap(pixmap) \
-{ \
-  pixmap.version=ReadBlobMSBShort(image); \
-  pixmap.pack_type=ReadBlobMSBShort(image); \
-  pixmap.pack_size=ReadBlobMSBLong(image); \
-  pixmap.horizontal_resolution=ReadBlobMSBLong(image); \
-  pixmap.vertical_resolution=ReadBlobMSBLong(image); \
-  pixmap.pixel_type=ReadBlobMSBShort(image); \
-  pixmap.bits_per_pixel=ReadBlobMSBShort(image); \
-  pixmap.component_count=ReadBlobMSBShort(image); \
-  pixmap.component_size=ReadBlobMSBShort(image); \
-  pixmap.plane_bytes=ReadBlobMSBLong(image); \
-  pixmap.table=ReadBlobMSBLong(image); \
-  pixmap.reserved=ReadBlobMSBLong(image); \
-  if (pixmap.bits_per_pixel <= 0 || pixmap.bits_per_pixel > 32 || \
-      pixmap.component_count <= 0 || pixmap.component_count > 4 || \
-      pixmap.component_size <= 0) \
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image); \
-}
+#define ReadPixmap(pixmap)                                             \
+  {                                                                    \
+  pixmap.version=ReadBlobMSBShort(image);                              \
+  pixmap.pack_type=ReadBlobMSBShort(image);                            \
+  pixmap.pack_size=ReadBlobMSBLong(image);                             \
+  pixmap.horizontal_resolution=ReadBlobMSBLong(image);                 \
+  pixmap.vertical_resolution=ReadBlobMSBLong(image);                   \
+  pixmap.pixel_type=ReadBlobMSBShort(image);                           \
+  pixmap.bits_per_pixel=ReadBlobMSBShort(image);                       \
+  pixmap.component_count=ReadBlobMSBShort(image);                      \
+  pixmap.component_size=ReadBlobMSBShort(image);                       \
+  pixmap.plane_bytes=ReadBlobMSBLong(image);                           \
+  pixmap.table=ReadBlobMSBLong(image);                                 \
+  pixmap.reserved=ReadBlobMSBLong(image);                              \
+  }
 
-#define ReadRectangle(rectangle) \
-{ \
-  rectangle.top=ReadBlobMSBShort(image); \
-  rectangle.left=ReadBlobMSBShort(image); \
-  rectangle.bottom=ReadBlobMSBShort(image); \
-  rectangle.right=ReadBlobMSBShort(image); \
-  if (rectangle.top > rectangle.bottom || \
-      rectangle.left > rectangle.right) \
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image); \
-}
+#define ValidatePixmap(pixmap) \
+  (!(EOFBlob(image) ||                                                 \
+     pixmap.bits_per_pixel <= 0 || pixmap.bits_per_pixel > 32 ||       \
+     pixmap.component_count <= 0 || pixmap.component_count > 4 ||      \
+     pixmap.component_size <= 0))
+
+#define ReadRectangle(rectangle)                                       \
+  {                                                                    \
+  rectangle.top=ReadBlobMSBShort(image);                               \
+  rectangle.left=ReadBlobMSBShort(image);                              \
+  rectangle.bottom=ReadBlobMSBShort(image);                            \
+  rectangle.right=ReadBlobMSBShort(image);                             \
+  }
+
+#define ValidateRectangle(rectangle) \
+  (!(EOFBlob(image) || \
+     (rectangle.top > rectangle.bottom || rectangle.left > rectangle.right)))
 
 typedef struct _PICTCode
 {
@@ -404,7 +407,7 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
   return(scanline);
 }
 
-static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
+static unsigned char *DecodeImage(const ImageInfo *image_info,
   Image *blob,Image *image,unsigned long bytes_per_line,
   const unsigned int bits_per_pixel)
 {
@@ -421,7 +424,6 @@ static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
 
   size_t
     allocated_pixels,
-    length,
     row_bytes;
 
   unsigned char
@@ -430,10 +432,19 @@ static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
 
   unsigned long
     bytes_per_pixel,
+    length,
     number_pixels,
     scanline_length,
     width;
 
+  ARG_NOT_USED(image_info);
+
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "DecodeImage: bytes_per_line=%lu,"
+                          " bits_per_pixel=%u",
+                          bytes_per_line, bits_per_pixel);
+
   /*
     Determine pixel buffer size.
   */
@@ -454,6 +465,12 @@ static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
   row_bytes=(size_t) (image->columns | 0x8000);
   if (image->storage_class == DirectClass)
     row_bytes=(size_t) ((4*image->columns) | 0x8000);
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "DecodeImage: Using %lu bytes per line, %"
+                          MAGICK_SIZE_T_F "u bytes per row",
+                          bytes_per_line,
+                          (MAGICK_SIZE_T) row_bytes);
   /*
     Allocate pixel and scanline buffer.
   */
@@ -474,7 +491,12 @@ static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
       {
         q=pixels+y*width;
         number_pixels=bytes_per_line;
-        (void) ReadBlob(blob,number_pixels,(char *) scanline);
+        if (ReadBlob(blob,number_pixels,(char *) scanline) != number_pixels)
+          {
+            ThrowException(&image->exception,CorruptImageError,UnexpectedEndOfFile,
+                           image->filename);
+            goto decode_error_exit;
+          }
         p=ExpandBuffer(scanline,&number_pixels,bits_per_pixel);
         (void) memcpy(q,p,number_pixels);
       }
@@ -495,23 +517,34 @@ static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
       {
         ThrowException(&image->exception,CorruptImageError,UnableToUncompressImage,
                        "scanline length exceeds row bytes");
-        break;
+        goto decode_error_exit;
+      }
+    if (ReadBlob(blob,scanline_length,(char *) scanline) != scanline_length)
+      {
+        ThrowException(&image->exception,CorruptImageError,UnexpectedEndOfFile,
+                       image->filename);
+        goto decode_error_exit;
       }
-    (void) ReadBlob(blob,scanline_length,(char *) scanline);
     for (j=0; j < (long) scanline_length; )
       if ((scanline[j] & 0x80) == 0)
         {
           length=(scanline[j] & 0xff)+1;
           number_pixels=length*bytes_per_pixel;
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
+          if (j+number_pixels >= scanline_length)
+            {
+              ThrowException(&image->exception,CorruptImageError,UnableToUncompressImage,
+                             "Decoded RLE pixels exceeds allocation!");
+              goto decode_error_exit;
+            }
           if ((q+number_pixels > pixels+allocated_pixels))
             {
               ThrowException(&image->exception,CorruptImageError,UnableToUncompressImage,
                              "Decoded RLE pixels exceeds allocation!");
-              break;
+              goto decode_error_exit;
             }
 
-          (void) memcpy(q,p,number_pixels);
+          (void) memcpy(q,p,number_pixels); /* ASAN report */
           q+=number_pixels;
           j+=length*bytes_per_pixel+1;
         }
@@ -526,7 +559,7 @@ static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
             {
               ThrowException(&image->exception,CorruptImageError,UnableToUncompressImage,
                              "Decoded RLE pixels exceeds allocation!");
-              break;
+              goto decode_error_exit;
             }
             (void) memcpy(q,p,number_pixels);
             q+=number_pixels;
@@ -536,6 +569,12 @@ static unsigned char *DecodeImage(const ImageInfo *ARGUNUSED(image_info),
   }
   MagickFreeMemory(scanline);
   return(pixels);
+
+ decode_error_exit:
+
+  MagickFreeMemory(scanline);
+  MagickFreeMemory(pixels);
+  return (unsigned char *) NULL;
 }
 
 /*
@@ -683,12 +722,12 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   length=(q-pixels);
   if (bytes_per_line > 200)
     {
-      (void) WriteBlobMSBShort(image,length);
+      (void) WriteBlobMSBShort(image,(const magick_uint16_t) length);
       length+=2;
     }
   else
     {
-      (void) WriteBlobByte(image,(long) length);
+      (void) WriteBlobByte(image,(const magick_uint8_t) length);
       length++;
     }
   while (q != pixels)
@@ -731,6 +770,13 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
 %
 %
 */
+#define ThrowPICTReaderException(code_,reason_,image_) \
+{ \
+  if (clone_info) \
+    DestroyImageInfo(clone_info); \
+  ThrowReaderException(code_,reason_,image_); \
+}
+
 static Image *ReadPICTImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -740,6 +786,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   Image
     *image;
 
+  ImageInfo
+    *clone_info = (ImageInfo *) NULL;
+
   IndexPacket
     index;
 
@@ -788,7 +837,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+    ThrowPICTReaderException(FileOpenError,UnableToOpenFile,image);
   pixmap.bits_per_pixel=0;
   pixmap.component_count=0;
   /*
@@ -800,26 +849,28 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     (void) ReadBlobByte(image);  /* skip header */
   (void) ReadBlobMSBShort(image);  /* skip picture size */
   ReadRectangle(frame);
+  if (!ValidateRectangle(frame))
+    ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
   while ((c=ReadBlobByte(image)) == 0);
   if (c != 0x11)
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+    ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
   version=ReadBlobByte(image);
   if (version == 2)
     {
       c=ReadBlobByte(image);
       if (c != 0xff)
-        ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+        ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
     }
   else
     if (version != 1)
-      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
   /*
     Validate dimensions
   */
   if ((frame.left < 0) || (frame.right < 0) || (frame.top < 0) || (frame.bottom < 0) ||
       (frame.left >= frame.right) || (frame.top >= frame.bottom))
     {
-      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
     }
 
   /*
@@ -833,6 +884,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                           "Dimensions: %lux%lu",image->columns,image->rows);
 
+  if (CheckImagePixelLimits(image, exception) != MagickPass)
+    ThrowPICTReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+
   /*
     Interpret PICT opcodes.
   */
@@ -871,11 +925,15 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             ReadRectangle(frame);
+            if (!ValidateRectangle(frame))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             if ((frame.left & 0x8000) || (frame.top & 0x8000))
               break;
             image->columns=frame.right-frame.left;
             image->rows=frame.bottom-frame.top;
-            (void) SetImage(image,OpaqueOpacity);
+            if (CheckImagePixelLimits(image, exception) != MagickPass)
+              ThrowPICTReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+            (void) SetImageEx(image,OpaqueOpacity,exception);
             break;
           }
           case 0x12:
@@ -902,11 +960,15 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (pattern != 1)
-              ThrowReaderException(CorruptImageError,UnknownPatternType,
+              ThrowPICTReaderException(CorruptImageError,UnknownPatternType,
                 image);
             length=ReadBlobMSBShort(image);
             ReadRectangle(frame);
+            if (!ValidateRectangle(frame))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             ReadPixmap(pixmap);
+            if (!ValidatePixmap(pixmap))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             (void) ReadBlobMSBLong(image);
             flags=ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
@@ -1007,6 +1069,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 (void) ReadBlobMSBShort(image);
               }
             ReadRectangle(frame);
+            if (!ValidateRectangle(frame))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             /*
               Initialize tile image.
             */
@@ -1019,6 +1083,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if ((code == 0x9a) || (code == 0x9b) || (bytes_per_line & 0x8000))
               {
                 ReadPixmap(pixmap);
+                if (!ValidatePixmap(pixmap))
+                  ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
                 tile_image->matte=pixmap.component_count == 4;
               }
             if ((code != 0x9a) && (code != 0x9b))
@@ -1036,7 +1102,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (!AllocateImageColormap(tile_image,tile_image->colors))
                   {
                     DestroyImage(tile_image);
-                    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image)
+                    ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image)
                   }
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "Allocated tile image colormap with %u colors",tile_image->colors);
@@ -1069,7 +1135,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   }
               }
             ReadRectangle(source);
+            if (!ValidateRectangle(source))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             ReadRectangle(destination);
+            if (!ValidateRectangle(destination))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             (void) ReadBlobMSBShort(image);
             if ((code == 0x91) || (code == 0x99) || (code == 0x9b))
               {
@@ -1088,8 +1158,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 pixmap.bits_per_pixel);
             if (pixels == (unsigned char *) NULL)
               {
+                CopyException(exception, &tile_image->exception);
                 DestroyImage(tile_image);
-                ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image)
+                ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image)
               }
             /*
               Convert PICT tile image to pixel packets.
@@ -1100,7 +1171,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               q=SetImagePixels(tile_image,0,y,tile_image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(tile_image);
+              indexes=AccessMutableIndexes(tile_image);
               for (x=0; x < (long) tile_image->columns; x++)
               {
                 if (tile_image->storage_class == PseudoClass)
@@ -1151,7 +1222,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 p+=(pixmap.component_count-1)*tile_image->columns;
               if (destination.bottom == (long) image->rows)
                 if (QuantumTick(y,tile_image->rows))
-                  if (!MagickMonitor(LoadImageText,y,tile_image->rows,&image->exception))
+                  if (!MagickMonitorFormatted(y,tile_image->rows,&image->exception,
+                                              LoadImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
             MagickFreeMemory(pixels);
@@ -1162,7 +1235,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                    destination.left,destination.top);
                 DestroyImage(tile_image);
             if (destination.bottom != (long) image->rows)
-              if (!MagickMonitor(LoadImageText,destination.bottom,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(destination.bottom,image->rows,&image->exception,
+                                          LoadImageText,image->filename,
+					  image->columns,image->rows))
                 break;
             break;
           }
@@ -1196,7 +1271,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (length == 0)
                   break;
                 if (SetImageProfile(image,"ICM",info,length) == MagickFail)
-                  ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                  ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image);
                 MagickFreeMemory(info);
                 break;
               }
@@ -1205,7 +1280,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (length == 0)
                   break;
                 if (SetImageProfile(image,"IPTC",info,length) == MagickFail)
-                  ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                  ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image);
                 MagickFreeMemory(info);
                 break;
               }
@@ -1248,9 +1323,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         Image
           *tile_image;
 
-        ImageInfo
-          *clone_info;
-
         /*
           Embedded JPEG.
         */
@@ -1265,20 +1337,31 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             ThrowReaderTemporaryFileException(clone_info->filename);
           }
         length=ReadBlobMSBLong(image);
-        for (i=0; i < 6; i++)
-          (void) ReadBlobMSBLong(image);
-        ReadRectangle(frame);
-        for (i=0; i < 122; i++)
-          (void) ReadBlobByte(image);
-        for (i=0; i < (long) (length-154); i++)
-        {
-          c=ReadBlobByte(image);
-          (void) fputc(c,file);
-        }
+        if (length > 154)
+          {
+            for (i=0; i < 6; i++)
+              (void) ReadBlobMSBLong(image);
+            ReadRectangle(frame);
+            if (!ValidateRectangle(frame))
+              {
+                (void) fclose(file);
+                ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
+              }
+            for (i=0; i < 122; i++)
+              if (ReadBlobByte(image) == EOF)
+                break;
+            for (i=0; i < (long) (length-154); i++)
+              {
+                if ((c=ReadBlobByte(image)) == EOF)
+                  break;
+                (void) fputc(c,file);
+              }
+          }
         (void) fclose(file);
         tile_image=ReadImage(clone_info,exception);
         (void) LiberateTemporaryFile(clone_info->filename);
         DestroyImageInfo(clone_info);
+        clone_info=(ImageInfo *) NULL;
         if (tile_image == (Image *) NULL)
           continue;
         FormatString(geometry,"%lux%lu",Max(image->columns,tile_image->columns),
@@ -1300,7 +1383,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         */
         length=ReadBlobMSBShort(image);
         for (i=0; i < (long) length; i++)
-          (void) ReadBlobByte(image);
+          if (ReadBlobByte(image) == EOF)
+            break;
         continue;
       }
     if ((code >= 0x100) && (code <= 0x7fff))
@@ -1310,7 +1394,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         */
         length=(code >> 7) & 0xff;
         for (i=0; i < (long) length; i++)
-          (void) ReadBlobByte(image);
+          if (ReadBlobByte(image) == EOF)
+            break;
         continue;
       }
   }
@@ -1353,15 +1438,18 @@ ModuleExport void RegisterPICTImage(void)
   entry->decoder=(DecoderHandler) ReadPICTImage;
   entry->encoder=(EncoderHandler) WritePICTImage;
   entry->adjoin=False;
-  entry->description=AcquireString("Apple Macintosh QuickDraw/PICT");
-  entry->module=AcquireString("PICT");
+  entry->description="Apple Macintosh QuickDraw/PICT";
+  entry->seekable_stream=MagickTrue;
+  entry->module="PICT";
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("PICT");
   entry->decoder=(DecoderHandler) ReadPICTImage;
   entry->encoder=(EncoderHandler) WritePICTImage;
   entry->adjoin=False;
-  entry->description=AcquireString("Apple Macintosh QuickDraw/PICT");
-  entry->module=AcquireString("PICT");
+  entry->description="Apple Macintosh QuickDraw/PICT";
+  entry->seekable_stream=MagickTrue;
+  entry->module="PICT";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1420,6 +1508,17 @@ ModuleExport void UnregisterPICTImage(void)
 %
 %
 */
+#define LiberatePICTAllocations()               \
+  {                                             \
+    MagickFreeMemory(buffer);                   \
+    MagickFreeMemory(packed_scanline);          \
+    MagickFreeMemory(scanline);                 \
+  }
+#define ThrowPICTWriterException(code_,reason_,image_)  \
+  {                                                     \
+    LiberatePICTAllocations();                          \
+    ThrowWriterException(code_,reason_,image_);         \
+  }
 static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
 {
 #define MaxCount  128U
@@ -1462,7 +1561,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
+  register const IndexPacket
     *indexes;
 
   register long
@@ -1470,23 +1569,23 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     x;
 
   size_t
-    count;
+    bytes_per_line,
+    count,
+    row_bytes;
 
   unsigned char
-    *buffer,
-    *packed_scanline,
-    *scanline;
+    *buffer = (unsigned char *) NULL,
+    *packed_scanline = (unsigned char *) NULL,
+    *scanline = (unsigned char *) NULL;
 
   unsigned int
     status;
 
   unsigned long
-    bytes_per_line,
     storage_class;
 
   unsigned short
     base_address,
-    row_bytes,
     transfer_mode;
 
   /*
@@ -1497,10 +1596,10 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if ((image->columns > 65535L) || (image->rows > 65535L))
-    ThrowWriterException(ImageError,WidthOrHeightExceedsLimit,image);
+    ThrowPICTWriterException(ImageError,WidthOrHeightExceedsLimit,image);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,UnableToOpenFile,image);
+    ThrowPICTWriterException(FileOpenError,UnableToOpenFile,image);
   (void) TransformColorspace(image,RGBColorspace);
   /*
     Initialize image info.
@@ -1514,7 +1613,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   source_rectangle=size_rectangle;
   destination_rectangle=size_rectangle;
   base_address=0xff;
-  row_bytes=(unsigned short) (image->columns | 0x8000);
+  row_bytes=image->columns;
   bounds.top=0;
   bounds.left=0;
   bounds.bottom=(short) image->rows;
@@ -1542,7 +1641,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
       pixmap.bits_per_pixel=32;
       pixmap.pack_type=0x04;
       transfer_mode=0x40;
-      row_bytes=(unsigned short) ((4*image->columns) | 0x8000);
+      row_bytes=4*image->columns;
     }
   /*
     Allocate memory.
@@ -1556,7 +1655,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   if ((buffer == (unsigned char *) NULL) ||
       (packed_scanline == (unsigned char *) NULL) ||
       (scanline == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+    ThrowPICTWriterException(ResourceLimitError,MemoryAllocationFailed,image);
   /*
     Write header, header size, size bounding box, version, and reserved.
   */
@@ -1591,7 +1690,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0x1f2);
-      (void) WriteBlobMSBShort(image,profile_length+4);
+      (void) WriteBlobMSBShort(image,(const magick_uint16_t) profile_length+4);
       (void) WriteBlobString(image,"8BIM");
       (void) WriteBlob(image,profile_length,
                        profile_info);
@@ -1604,7 +1703,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
-      (void) WriteBlobMSBShort(image,profile_length+4);
+      (void) WriteBlobMSBShort(image,(const magick_uint16_t) profile_length+4);
       (void) WriteBlobMSBLong(image,0x00000000UL);
       (void) WriteBlob(image,profile_length,
                        profile_info);
@@ -1636,6 +1735,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
       jpeg_image=CloneImage(image,0,0,True,&image->exception);
       if (jpeg_image == (Image *) NULL)
         {
+          LiberatePICTAllocations();
           CloseBlob(image);
           return (False);
         }
@@ -1646,9 +1746,13 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
         &image->exception);
       DestroyImage(jpeg_image);
       if (blob == (unsigned char *) NULL)
-        return(False);
+        {
+          LiberatePICTAllocations();
+          CloseBlob(image);
+          return(False);
+        }
       (void) WriteBlobMSBShort(image,PictJPEGOp);
-      (void) WriteBlobMSBLong(image,length+154);
+      (void) WriteBlobMSBLong(image,(const magick_uint16_t) length+154);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBLong(image,0x00010000UL);
       (void) WriteBlobMSBLong(image,0x00000000UL);
@@ -1775,12 +1879,14 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
+      indexes=AccessImmutableIndexes(image);
       for (x=0; x < (long) image->columns; x++)
         scanline[x]=indexes[x];
       count+=EncodeImage(image,scanline,row_bytes & 0x7FFF,packed_scanline);
       if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+        if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                    SaveImageText,image->filename,
+				    image->columns,image->rows))
           break;
     }
   else
@@ -1828,7 +1934,9 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
           }
           count+=EncodeImage(image,scanline,bytes_per_line & 0x7FFF,packed_scanline);
           if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+            if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                        SaveImageText,image->filename,
+					image->columns,image->rows))
               break;
         }
       }
