@@ -41,6 +41,7 @@ extern MagickExport void
   DestroyImageList(Image *),
   InsertImageInList(Image **,Image *),
   PrependImageToList(Image **,Image *),
+  ReplaceImageInList(Image **images,Image *image),
   ReverseImageList(Image **),
   SpliceImageIntoList(Image **,const unsigned long,Image *);
 
@@ -49,3 +50,11 @@ extern MagickExport void
 #endif
 
 #endif
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 2
+ * fill-column: 78
+ * End:
+ */
