@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -92,15 +92,19 @@ MagickExport unsigned int ExecuteModuleProcess(const char *tag,
 
   if (method)
     {
-      LogMagickEvent(CoderEvent,GetMagickModule(),
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "Invoking \"%.1024s\" filter module",tag);
 
       status=(*method)(image,argc,argv);
 
-      LogMagickEvent(CoderEvent,GetMagickModule(),
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "Returned from \"%.1024s\" filter module",tag);
     }
-
+#else
+  ARG_NOT_USED(tag);
+  ARG_NOT_USED(image);
+  ARG_NOT_USED(argc);
+  ARG_NOT_USED(argv);
 #endif /* !defined(BuildMagickModules) */
   return(status);
 }
@@ -116,7 +120,8 @@ MagickExport unsigned int ExecuteModuleProcess(const char *tag,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterStaticModules() statically registers all the available module
+%  
+RegisterStaticModules() statically registers all the available module
 %  handlers.
 %
 %  The format of the RegisterStaticModules method is:
@@ -129,7 +134,6 @@ MagickExport void RegisterStaticModules(void)
 {
 #if !defined(BuildMagickModules)
   RegisterARTImage();
-  RegisterAVIImage();
   RegisterAVSImage();
   RegisterBMPImage();
   RegisterCALSImage();
@@ -166,9 +170,13 @@ MagickExport void RegisterStaticModules(void)
   RegisterHTMLImage();
   RegisterICONImage();
   RegisterIDENTITYImage();
+  RegisterINFOImage();
 #if defined(HasJBIG)
   RegisterJBIGImage();
 #endif
+#if defined(HasJPEG)
+  RegisterJNXImage();
+#endif
 #if defined(HasJPEG)
   RegisterJPEGImage();
 #endif
@@ -178,6 +186,7 @@ MagickExport void RegisterStaticModules(void)
   RegisterLABELImage();
   RegisterLOCALEImage();
   RegisterLOGOImage();
+  RegisterMACImage();
   RegisterMAPImage();
   RegisterMATImage();
   RegisterMATTEImage();
@@ -209,7 +218,9 @@ MagickExport void RegisterStaticModules(void)
   RegisterPSImage();
   RegisterPS2Image();
   RegisterPS3Image();
+#if defined(EnableBrokenCoders) && EnableBrokenCoders
   RegisterPSDImage();
+#endif /* defined(EnableBrokenCoders) && EnableBrokenCoders */
   RegisterPWPImage();
   RegisterRGBImage();
   RegisterRLAImage();
@@ -236,6 +247,9 @@ MagickExport void RegisterStaticModules(void)
   RegisterVIDImage();
   RegisterVIFFImage();
   RegisterWBMPImage();
+#if defined(HasWEBP)
+  RegisterWEBPImage();
+#endif
   RegisterWMFImage();
   RegisterWPGImage();
 #if defined(HasX11)
@@ -279,7 +293,6 @@ MagickExport void UnregisterStaticModules(void)
 {
 #if !defined(BuildMagickModules)
   UnregisterARTImage();
-  UnregisterAVIImage();
   UnregisterAVSImage();
   UnregisterBMPImage();
   UnregisterCALSImage();
@@ -316,9 +329,13 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterHTMLImage();
   UnregisterICONImage();
   UnregisterIDENTITYImage();
+  UnregisterINFOImage();
 #if defined(HasJBIG)
   UnregisterJBIGImage();
 #endif
+#if defined(HasJPEG)
+  UnregisterJNXImage();
+#endif
 #if defined(HasJPEG)
   UnregisterJPEGImage();
 #endif
@@ -328,6 +345,7 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterLABELImage();
   UnregisterLOCALEImage();
   UnregisterLOGOImage();
+  UnregisterMACImage();
   UnregisterMAPImage();
   UnregisterMATImage();
   UnregisterMATTEImage();
@@ -359,7 +377,9 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterPSImage();
   UnregisterPS2Image();
   UnregisterPS3Image();
+#if defined(EnableBrokenCoders) && EnableBrokenCoders
   UnregisterPSDImage();
+#endif /* defined(EnableBrokenCoders) && EnableBrokenCoders */
   UnregisterPWPImage();
   UnregisterRGBImage();
   UnregisterRLAImage();
@@ -386,6 +406,9 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterVIDImage();
   UnregisterVIFFImage();
   UnregisterWBMPImage();
+#if defined(HasWEBP)
+  UnregisterWEBPImage();
+#endif
   UnregisterWMFImage();
   UnregisterWPGImage();
 #if defined(HasX11)
